/*
 * Copyright (C) 2013 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by Pawel Stolowski <pawel.stolowski@canonical.com>
 *
 */

namespace Unity.HomeScope.SmartScopes {

  public class PlatformInfo
  {
    public string platform { get; set; }
    public string locale { get; set; }
    public string[] added_scopes { get; set; }
    public string[] removed_scopes { get; set; }

    private PlatformInfo ()
    {
    }

    public PlatformInfo.with_data (string platform, string? locale, string[] added, string[] removed)
    {
      this.platform = platform;
      this.locale = locale;
      this.added_scopes = added;
      this.removed_scopes = removed;
    }

    public static PlatformInfo gather_platform_info (ClientScopesInfo? client_scopes_info)
    {
      var info = new PlatformInfo ();
      info.platform = get_release_string ();

      var languages = GLib.Intl.get_language_names ();
      info.locale = languages[0]; //list is sorted from most desirable to least desirable, pick the first one

      if (client_scopes_info != null)
      {
        info.added_scopes = client_scopes_info.get_added_scopes ().to_array ();
        info.removed_scopes = client_scopes_info.get_removed_scopes ().to_array ();
      }

      return info;
    }

    internal static string get_release_string ()
    {
      // FIXME: we don't have form factor info yet, so for now return "unknown-${version}",
      // where version is obtained from lsb_release.
      // ubuntu_version tool needs to be used instead when it's available.
      try
      {
        string data;
        if (GLib.Process.spawn_sync (null, {"lsb_release", "-s", "-r"}, null, SpawnFlags.SEARCH_PATH , null, out data, null, null))
        {
          if (data != null)
            return "unknown-" + data.strip ();
        }
      }
      catch (Error e)
      {
        warning ("Failed to process lsb-release info: %s", e.message);
      }
      return "unknown";
    }
  }
}
