/*
 * Copyright (C) 2012 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by Pawel Stolowski <pawel.stolowski@canonical.com>
 *
 */

namespace Unity.HomeScope {

public class ScopeManager
{
  private PreferencesManager preferences = PreferencesManager.get_default ();
  private MasterScopesManager master_scopes_mgr = new MasterScopesManager ();
  private Gee.Set<string> client_scopes = new Gee.HashSet<string> ();

  public ScopeManager ()
  {
  }

  public void start_master_scopes ()
  {
    master_scopes_mgr.start ();
  }

  public bool is_client_scope (string scope_id)
  {
    if (client_scopes.size == 0)
      ScopeRegistry.instance ().flatten ();
    return client_scopes.contains (scope_id);
  }

  /**
   * Returns true if scope is not a client scope; shouldn't be called for master scopes.
   */
  public bool is_server_scope (string scope_id)
  {
    var reg = ScopeRegistry.instance ();
    if (client_scopes == null)
    {
      foreach (var id in reg.flatten ())
      {
        if (!reg.is_master (id)) // ignore master scopes
          client_scopes.add (id);
      }
    }
    return client_scopes.contains (scope_id) == false;
  }

  /**
   * Get ids of scopes with always-run flag on
   */
  public Gee.Set<string> get_always_run_scopes ()
  {
    var scopes = new Gee.TreeSet<string> ();
    foreach (string scope in preferences.always_search)
    {
      // only return scope if its in registry
      foreach (var topscope in ScopeRegistry.instance ().scopes)
      {
        if (topscope.scope_info.id == scope)
        {
          scopes.add (scope);
          break;
        }

        foreach (var subscope in topscope.sub_scopes)
        {
          if (subscope.id == scope)
          {
            scopes.add (scope);
            break;
          } 
        }
      }
    }
    return scopes;
  }
}

}
