/*
 * Copyright (C) 2013 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by Pawel Stolowski <pawel.stolowski@canonical.com>
 *
 */

namespace Unity.HomeScope {

  public enum SearchQueryChange
  {
    NOT_CHANGED,
    EMPTY_QUERY,
    NEW_QUERY,
    APPENDS_TO_PREVIOUS_QUERY,
    REMOVES_FROM_PREVIOUS_QUERY
  }
 
  public class SearchQueryState
  {
    private HashTable<string, string> previous_query = new HashTable<string, string> (str_hash, str_equal);
    
    public void remove (string home_channel_id)
    {
      previous_query.remove (home_channel_id);
    }

    public bool has_channel (string home_channel_id)
    {
      return previous_query.contains (home_channel_id);
    }

    /**
     * Compare search string with previous string for given home channel. Store new search string.
     */
    public SearchQueryChange search_query_changed (string channel_id, string search_string)
    {
      var changed = SearchQueryChange.NEW_QUERY;
      if (previous_query.contains (channel_id))
      {
        if (search_string == "") // there was a query previously, but user removed all characters in new one
        {
          previous_query.remove (channel_id); // forget previous query
          changed = SearchQueryChange.EMPTY_QUERY;
        }
        else
        {
          var prev = previous_query[channel_id];
          if (prev == search_string)
            changed = SearchQueryChange.NOT_CHANGED;
          else if (search_string.has_prefix (prev))
            changed = SearchQueryChange.APPENDS_TO_PREVIOUS_QUERY;
          else if (prev.has_prefix (search_string))
            changed = SearchQueryChange.REMOVES_FROM_PREVIOUS_QUERY;
        }
      }
      else
      {
        if (search_string == "")
          changed = SearchQueryChange.EMPTY_QUERY;
      }

      if (changed != SearchQueryChange.EMPTY_QUERY)
        previous_query[channel_id] = search_string;

      debug ("search_query_changed for channel %s: %s", channel_id, changed.to_string ());
      return changed;
    }
  }
}

